from django.utils import simplejson
from django import forms
from django.forms.util import flatatt
from django.utils import safestring
from django.template.loader import render_to_string

__all__ = ['AutocompleteWidget']


class AutocompleteWidget(forms.SelectMultiple):
    """
    Widget suitable for ModelChoiceField and ModelMultipleChoiceField.

    Example usage::

        from django import forms

        import autocomplete_light

        from models import Author

        class AuthorsForm(forms.Form):
            lead_author = forms.ModelChoiceField(Author.objects.all(), widget=
                autocomplete_light.AutocompleteWidget(
                    'AuthorChannel', max_items=1))

            contributors = forms.ModelMultipleChoiceField(Author.objects.all(),
                widget=autocomplete_light.AutocompleteWidget('AuthorChannel'))
    """

    payload = {}

    def __init__(self, channel_name, *args, **kwargs):
        """
        AutocompleteWidget constructor decorates SelectMultiple constructor

        Arguments:
        channel_name -- the name of the channel that this widget should use.

        Keyword arguments are passed to javascript via data attributes of the
        autocomplete wrapper element:

        max_items
            The number of items that this autocomplete allows. If set to 0,
            then it allows any number of selected items like a multiple select,
            well suited for ManyToMany relations or any kind of
            ModelMultipleChoiceField. If set to 3 for example, then it will
            only allow 3 selected items. It should be set to 1 if the widget is
            for a ModelChoiceField or ForeignKey, in that case it would be like
            a normal select. Default is 0.

        min_characters
            The minimum number of characters before the autocomplete box shows
            up. If set to 2 for example, then the autocomplete box will show up
            when the input receives the second character, for example 'ae'. If
            set to 0, then the autocomplete box will show up as soon as the
            input is focused, even if it's empty, behaving like a normal
            select. Default is 0.

        bootstrap
            The name of the bootstrap kind. By default, deck.js will only
            initialize decks for wrappers that have data-bootstrap="normal". If
            you want to implement your own bootstrapping logic in javascript,
            then you set bootstrap to anything that is not "normal". By
            default, its value is copied from channel.bootstrap.

        placeholder
            The initial value of the autocomplete input field. It can be
            something like 'type your search here'. By default, it is copied
            from channel.placeholder.

        payload
            A dict of data that will be exported to JSON, and parsed into the
            Deck instance in javascript. It allows to pass variables from
            Python to Javascript.
        """
        self.channel_name = channel_name

        from autocomplete_light import registry
        self.channel = registry[channel_name]()

        self.payload.update(kwargs.pop('payload', {}))
        self.max_items = kwargs.pop('max_items', 0)
        self.min_characters = kwargs.pop('min_characters', 0)
        self.bootstrap = kwargs.pop('bootstrap', self.channel.bootstrap)
        self.placeholder = kwargs.pop('placeholder', self.channel.placeholder)

        super(AutocompleteWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        """
        Render the autocomplete widget.

        It will try two templates, like django admin:
        - autocomplete_light/channelname/widget.html
        - autocomplete_light/widget.html

        Note that it will not pass 'value' to the template, because 'value'
        might be a list of model ids in the case of ModelMultipleChoiceField,
        or a model id in the case of ModelChoiceField. To keep things simple,
        it will just pass a list, 'values', to the template context.
        """
        final_attrs = self.build_attrs(attrs)
        self.html_id = final_attrs.pop('id', name)

        if value and not isinstance(value, (list, tuple)):
            values = [value]
        else:
            values = value

        if values and not self.channel.are_valid(values):
            raise forms.ValidationError('%s cannot validate %s' % (
                self.channel_name, values))

        self.payload.update(self.as_dict())
        self.payload['channel'] = self.channel.as_dict()

        channel_name = self.channel_name.lower()
        return safestring.mark_safe(render_to_string([
                'autocomplete_light/%s/widget.html' % channel_name,
                'autocomplete_light/widget.html',
            ], {
                'widget': self,
                'name': name,
                'values': values,
                'channel': self.channel,
                'results': self.channel.get_results(values or []),
                'json_payload': safestring.mark_safe(simplejson.dumps(
                    self.payload)),
                'extra_attrs': safestring.mark_safe(flatatt(final_attrs)),
            }
        ))

    def as_dict(self):
        return {
            'max_items': self.max_items,
            'min_characters': self.min_characters,
            'bootstrap': self.bootstrap,
            # cast to unicode as it might be a gettext proxy
            'placeholder': unicode(self.placeholder),
        }

    # we might want to split up in two widgets for that ... is it necessary ?
    # apparently not yet, but maybe at next django release
    def value_from_datadict(self, data, files, name):
        """Route to Select if max_items is 1, else route to SelectMultiple"""
        if self.max_items == 1:
            return forms.Select.value_from_datadict(self, data, files, name)
        else:
            return forms.SelectMultiple.value_from_datadict(self, data, files,
                name)

    def _has_changed(self, initial, data):
        """Route to Select if max_items is 1, else route to SelectMultiple"""
        if self.max_items == 1:
            return forms.Select._has_changed(self, initial, data)
        else:
            return forms.SelectMultiple._has_changed(self, initial, data)



"""
An url to ChannelView.

autocomplete_light_channel
    Given a 'channel' argument with the name of the channel, this url routes to
    ChannelView.
"""

from django.conf.urls.defaults import patterns, url
from django.views.decorators.csrf import csrf_exempt

from views import ChannelView

urlpatterns = patterns('',
    url(r'^channel/(?P<channel>[-\w]+)/$',
        csrf_exempt(ChannelView.as_view()),
        name='autocomplete_light_channel'
    ),
)



from django.utils import simplejson

from base import ChannelBase

__all__ = ('JSONChannelBase', )


class JSONChannelBase(ChannelBase):
    """
    Example channel that attaches JSON to results.
    """
    textarea = u'<textarea style="display:none">%s</textarea>'

    def result_as_html(self, result, extra_context=None):
        """
        Adds the return value of result_as_json() to the context.
        """
        return super(JSONChannelBase, self).result_as_html(result, {
            'extra_html': self.textarea % self.result_as_json(result)})

    def result_as_dict(self, result):
        """
        Return the dict that should be passed as JSON for a result. Used by
        result_as_json().
        """
        return {'value': result.pk}

    def result_as_json(self, result):
        """
        Return the JSON to attach to a result, uses result_as_dict() by
        default.
        """
        return simplejson.dumps(self.result_as_dict(result))



import unittest

from django.db import models

import autocomplete_light
from autocomplete_light.templatetags import autocomplete_light_tags


class Foo(models.Model):
    pass


class Bar(autocomplete_light.ChannelBase):
    pass

class RegistryTestCase(unittest.TestCase):
    def setUp(self):
        self.registry = autocomplete_light.ChannelRegistry()

    def test_register_model(self):
        self.registry.register(Foo)
        self.assertIn('FooChannel', self.registry.keys())

    def test_register_model_and_channel(self):
        self.registry.register(Foo, Bar)
        self.assertIn('FooChannel', self.registry.keys())

    def test_register_channel(self):
        self.registry.register(Bar)
        self.assertIn('Bar', self.registry.keys())

    def test_unregister(self):
        self.registry.register(Bar)
        self.registry.unregister('Bar')
        self.assertEqual(self.registry.keys(), [])

    def test_register_with_kwargs(self):
        self.registry.register(Foo, search_name='search_name')
        self.assertEqual(self.registry['FooChannel'].search_name,
            'search_name')

    def test_register_with_channel_and_kwargs(self):
        self.registry.register(Foo, Bar, search_name='search_name')
        self.assertEqual(self.registry['FooChannel'].search_name,
            'search_name')





"""
The registry module provides tools to maintain a registry of channels.

The first thing that should happen when django starts is registration of
channels. It should happen first, because channels are required for
autocomplete widgets. And autocomplete widgets are required for forms. And
forms are required for ModelAdmin.

It looks like this:

- in ``yourapp/autocomplete_light_registry.py``, register your channels with
  ``autocomplete_light.register()``,
- in ``urls.py``, do ``autocomplete_light.autodiscover()`` **before**
  ``admin.autodiscover()``.

ChannelRegistry
    Subclass of Python's dict type with registration/unregistration methods.

registry
    Instance of ChannelRegistry.

register
    Proxy registry.register.

autodiscover
    Find channels and fill registry.
"""

from django.db import models

from .channel import ChannelBase

__all__ = ('ChannelRegistry', 'registry', 'register', 'autodiscover')


class ChannelRegistry(dict):
    """
    Dict with some shortcuts to handle a registry of channels.
    """

    def __init__(self):
        self._models = {}

    def channel_for_model(self, model):
        """Return the channel class for a given model."""
        try:
            return self._models[model]
        except KeyError:
            return

    def unregister(self, name):
        """
        Unregister a channel.
        """
        channel = self[name]
        del self[name]

        if channel.model:
            del self._models[channel.model]

    def register(self, *args, **kwargs):
        """
        Proxy registry.register_model_channel() or registry.register_channel()
        if there is no apparent model for the channel.

        Example usages::

            # Will create and register SomeModelChannel, if SomeChannel.model
            # is None (which is the case by default):
            autocomplete_light.register(SomeModel)

            # Same but using SomeChannel as base:
            autocomplete_light.register(SomeModel, SomeChannel)

            # Register a channel without model, ensure that SomeChannel.model
            # is None (which is the default):
            autocomplete_light.register(SomeChannel)

            # As of 0.5, you may also pass attributes*, ie.:
            autocomplete_light.register(SomeModel, search_field='search_names',
                result_template='somemodel_result.html')

        You may pass attributes via kwargs, only if the registry creates a
        type:

        - if no channel class is passed,
        - or if the channel class has no model attribute,
        - and if the channel classs is not generic
        """

        channel = None
        model = None

        for arg in args:
            if issubclass(arg, models.Model):
                model = arg
            elif issubclass(arg, ChannelBase):
                channel = arg

        if channel and getattr(channel, 'model'):
            model = channel.model

        if model:
            self.register_model_channel(model, channel, **kwargs)
        else:
            self.register_channel(channel)

    def register_model_channel(self, model, channel=None, **kwargs):
        """
        Add a model to the registry, optionnaly with a given channel class.

        model
            The model class to register.

        channel
            The channel class to register the model with, default to
            ChannelBase.

        kwargs
            Extra attributes to set to the channel class, if created by this
            method.

        Three cases are possible:

        - specify model class and ModelNameChannel will be generated extending
          ChannelBase, with attribute model=model
        - specify a model and a channel class that does not have a model
          attribute, and a ModelNameChannel will be generated, with attribute
          model=model
        - specify a model and a channel class with a model attribute, and the
          channel is directly registered

        To keep things simple, the name of a channel is it's class name.
        """
        kwargs.update({'model': model})

        if channel is None:
            channel = type('%sChannel' % model.__name__, (ChannelBase,),
                kwargs)
        elif channel.model is None:
            channel = type('%sChannel' % model.__name__, (channel,),
                kwargs)

        self.register_channel(channel)
        self._models[channel.model] = channel

    def register_channel(self, channel):
        """
        Register a channel without model, like a generic channel.
        """
        self[channel.__name__] = channel


def _autodiscover(registry):
    """See documentation for autodiscover (without the underscore)"""
    import copy
    from django.conf import settings
    from django.utils.importlib import import_module
    from django.utils.module_loading import module_has_submodule

    for app in settings.INSTALLED_APPS:
        mod = import_module(app)
        # Attempt to import the app's admin module.
        try:
            before_import_registry = copy.copy(registry)
            import_module('%s.autocomplete_light_registry' % app)
        except:
            # Reset the model registry to the state before the last import as
            # this import will have to reoccur on the next request and this
            # could raise NotRegistered and AlreadyRegistered exceptions
            # (see #8245).
            registry = before_import_registry

            # Decide whether to bubble up this error. If the app just
            # doesn't have an admin module, we can ignore the error
            # attempting to import it, otherwise we want it to bubble up.
            if module_has_submodule(mod, 'autocomplete_light_registry'):
                raise

registry = ChannelRegistry()


def autodiscover():
    """
    Check all apps in INSTALLED_APPS for stuff related to autocomplete_light.

    For each app, autodiscover imports app.autocomplete_light_registry if
    available, resulting in execution of register() statements in that module,
    filling registry.

    Consider a standard app called 'cities_light' with such a structure::

        cities_light/
            __init__.py
            models.py
            urls.py
            views.py
            autocomplete_light_registry.py

    With such a autocomplete_light_registry.py::

        from models import City, Country
        import autocomplete_light
        autocomplete_light.register(City)
        autocomplete_light.register(Country)

    When autodiscover() imports cities_light.autocomplete_light_registry, both
    CityChannel and CountryChannel will be registered. For details on how these
    channel classes are generated, read the documentation of
    ChannelRegistry.register.
    """
    _autodiscover(registry)


def register(*args, **kwargs):
    """Proxy registry.register"""
    return registry.register(*args, **kwargs)



from django import forms
from django.db import models
from django.forms import fields
from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

__all__ = ('GenericModelForm', 'GenericForeignKeyField')


class GenericModelForm(forms.ModelForm):
    """
    This simple subclass of ModelForm fixes a couple of issues with django's
    ModelForm.

    - treat virtual fields like GenericForeignKey as normal fields, Django
      should already do that but it doesn't,
    - when setting a GenericForeignKey value, also set the object id and
      content type id fields, again Django could probably afford to do that.
    """
    def __init__(self, *args, **kwargs):
        """
        What ModelForm does, but also add virtual field values to self.initial.
        """
        super(GenericModelForm, self).__init__(*args, **kwargs)

        # do what model_to_dict doesn't
        for field in self._meta.model._meta.virtual_fields:
            self.initial[field.name] = getattr(self.instance, field.name, None)

    def _post_clean(self):
        """
        What ModelForm does, but also set virtual field values from
        cleaned_data.
        """
        super(GenericModelForm, self)._post_clean()

        # take care of virtual fields since django doesn't
        for field in self._meta.model._meta.virtual_fields:
            value = self.cleaned_data.get(field.name, None)

            if value:
                setattr(self.instance, field.name, value)

    def save(self, commit=True):
        """
        What ModelForm does, but also set GFK.ct_field and GFK.fk_field if such
        a virtual field has a value.

        This should probably be done in the GFK field itself, but it's here for
        convenience until Django fixes that.
        """
        for field in self._meta.model._meta.virtual_fields:
            if isinstance(field, GenericForeignKey):
                value = self.cleaned_data.get(field.name, None)

                if not value:
                    continue

                setattr(self.instance, field.ct_field,
                    ContentType.objects.get_for_model(value))
                setattr(self.instance, field.fk_field, value.pk)

        return super(GenericModelForm, self).save(commit)


class GenericForeignKeyField(fields.Field):
    """
    Simple form field that converts strings to models.
    """

    def prepare_value(self, value):
        """
        Given a model instance as value, with content type id of 3 and pk of 5,
        return such a string '3-5'.
        """
        if isinstance(value, (str, unicode)):
            # Apparently there's a bug in django, that causes a python value to
            # be passed here. This ONLY happens when in an inline ....
            return value
        elif isinstance(value, models.Model):
            return '%s-%s' % (ContentType.objects.get_for_model(value).pk,
                value.pk)

    def to_python(self, value):
        """
        Given a string like '3-5', return the model of content type id 3 and pk
        5.
        """
        if not value:
            return value

        content_type_id, object_id = value.split('-')
        model = ContentType.objects.get_for_id(content_type_id).model_class()
        return model.objects.get(pk=object_id)



import urllib

from django import http
from django.utils import simplejson
from django.template import defaultfilters

from json import JSONChannelBase

__all__ = ['RemoteChannelBase', ]


class RemoteChannelBase(JSONChannelBase):
    """
    Uses an API to propose suggestions from an HTTP API, tested with
    djangorestframework.

    model_for_source_url
        A **very important function to override!** take an API URL and return
        the corresponding model class. This is API specific, there is a
        complete example in cities_light.contrib.

    source_url
        The full URL to the list API. For example, to a djangorestframework
        list view.

    An example implementation usage is demonstrated in the django-cities-light
    contrib folder.

    Autocomplete box display chronology:

    - autocomplete.js requests autocomplete box to display for an input,
    - get_results() fetches some extra results via get_remote_results(),
    - get_remote_results() calls source_url and returns a list of models,
    - the remote results are rendered after the local results in widget.html.
      It includes some JSON in a hidden textarea, like the API's url for each
      result.

    Remote result selection chronology:

    - deck.js calls remoteGetValue() instead of the default getValue(),
    - remoteGetValue() posts the json from the result to ChannelView,
    - ChannelView.post() does its job of proxying RemoteChannelBase.post(),
    - RemoteChannelBase.post() returns an http response which body is just the
      pk of the result in the local database, using self.fetch_result(),
    - self.fetch_result() passes the API url of the result and recursively
      saves the remote models into the local database, returning the id of the
      newly created object.
    """

    bootstrap = 'remote'

    def post(self, request, *args, **kwargs):
        """
        Take POST variable 'result', install it in the local database, return
        the newly created id.

        The HTTP response has status code 201 Created.
        """
        result = simplejson.loads(request.POST['result'])
        pk = self.fetch_result(result)
        return http.HttpResponse(pk, status=201)

    def model_for_source_url(self, url):
        """
        Take an URL from the API this remote channel is supposed to work with,
        return the model class to use for that url.

        It is only needed for the default implementation of fetch(), because it
        has to follow relations recursively.
        """
        raise NotImplemented()

    def fetch_result(self, result):
        """
        Take a result's dict representation, return it's local pk which might
        have been just created.

        If your channel works with 0 to 1 API call, consider overriding this
        method.
        If your channel is susceptible of using several different API calls,
        consider overriding fetch().
        """
        return self.fetch(result['source_url']).pk

    def fetch(self, url):
        """
        Given an url to a remote object, return the corresponding model from
        the local database.

        The default implementation expects url to respond with a JSON dict of
        the attributes of an object.

        For relation attributes, it expect the value to be another url that
        will respond with a JSON dict of the attributes of the related object.

        It calls model_for_source_url() to find which model class corresponds
        to which url. This allows fetch() to be recursive.
        """
        model_class = self.model_for_source_url(url)

        fh = urllib.urlopen(url)
        data = simplejson.loads(fh.read())
        data.pop('url')
        fh.close()

        for key, value in data.items():
            is_string = isinstance(value, (unicode, str))
            field = model_class._meta.get_field_by_name(key)[0]

            if getattr(field, 'rel', None) and is_string:
                data[key] = self.fetch(value)

        model, created = model_class.objects.get_or_create(**data)
        return model

    def get_results(self, values=None):
        """
        Returns a list of results from both the local database and the API if
        in the context of a request.

        Using self.limit_results and the number of local results, adds results
        from get_remote_results().
        """
        results = super(RemoteChannelBase, self).get_results(values)
        unicodes = [unicode(result) for result in results]

        if self.request:
            room = self.limit_results - len(results)

            if room > 0:
                results = list(results)

                for result in self.get_remote_results(room):
                    # avoid data that's already in local
                    if unicode(result) in unicodes:
                        continue

                    results.append(result)

        return results

    def get_remote_results(self, max):
        """
        Parses JSON from the API, return model instances.

        The JSON should contain a list of dicts. Each dict should contain the
        attributes of an object. Relation attributes should be represented by
        their url in the API, which is set to model._source_url.
        """
        url = self.get_source_url(max)

        try:
            fh = urllib.urlopen(url)
            body = fh.read()
        except:
            return
        else:
            for data in simplejson.loads(body):
                url = data.pop('url')

                for name in data.keys():
                    field = self.model._meta.get_field_by_name(name)[0]
                    if getattr(field, 'rel', None):
                        data.pop(name)
                model = self.model(**data)
                model._source_url = url
                yield model

    def result_as_value(self, result):
        """
        Return the result pk or source url.
        """
        result_url = getattr(result, '_source_url', None)

        if result_url:
            return defaultfilters.slugify(result_url)
        else:
            return result.pk

    def result_as_dict(self, result):
        """
        Return the result pk or _source_url.
        """
        result_url = getattr(result, '_source_url', None)

        if result_url:
            return {'source_url': result_url}
        else:
            return {'value': result.pk}

    def get_source_url(self, limit):
        """
        Return an API url for the current autocomplete request.

        By default, return self.source_url with the data dict returned by
        get_source_url_data().
        """
        return '%s?%s' % (self.source_url, urllib.urlencode(
            self.get_source_url_data(limit)))

    def get_source_url_data(self, limit):
        """
        Given a limit of items, return a dict of data to send to the API.

        By default, it passes current request GET arguments, along with format:
        'json' and the limit.
        """
        data = {}
        if self.request:
            for key, value in self.request.GET.items():
                data[key] = value

        data.update({
            'format': 'json',
            'limit': limit,
        })

        return data



"""
The channel.base module provides a channel class which you can extend to make
your own channel. It also serves as default channel class.
"""

from django.core import urlresolvers
from django.template import loader
from django.utils.translation import ugettext_lazy as _

__all__ = ('ChannelBase',)


class ChannelBase(object):
    """
    A basic implementation of a channel, which should fit most use cases.

    Attributes:

    model
        The model class this channel serves. If None, a new class will be
        created in registry.register, and the model attribute will be set in
        that subclass. So you probably don't need to worry about it, just know
        that it's there for you to use.

    result_template
        The template to use in result_as_html method, to render a single
        autocomplete suggestion. By default, it is
        autocomplete_light/channelname/result.html or
        autocomplete_light/result.html.

    autocomplete_template
        The template to use in render_autocomplete method, to render the
        autocomplete box. By default, it is
        autocomplete_light/channelname/autocomplete.html or
        autocomplete_light/autocomplete.html.

    search_field
        The name of the field that the default implementation of query_filter
        uses. Default is 'name'.

    limit_results
        The number of results that this channel should return. For example, if
        query_filter returns 50 results and that limit_results is 20, then the
        first 20 of 50 results will be rendered. Default is 20.

    bootstrap
        The name of the bootstrap kind. By default, deck.js will only
        initialize decks for wrappers that have data-bootstrap="normal". If
        you want to implement your own bootstrapping logic in javascript,
        then you set bootstrap to anything that is not "normal". Default is
        'normal'.

    placeholder
        The initial text in the autocomplete text input.
    """

    model = None
    search_field = 'name'
    limit_results = 20
    bootstrap = 'normal'
    placeholder = _(u'type some text to search in this autocomplete')
    result_template = None
    autocomplete_template = None

    def __init__(self):
        """
        Set result_template and autocomplete_template if necessary.
        """
        name = self.__class__.__name__.lower()
        if not self.result_template:
            self.result_template = [
                'autocomplete_light/%s/result.html' % name,
                'autocomplete_light/result.html',
            ]

        if not self.autocomplete_template:
            self.autocomplete_template = [
                'autocomplete_light/%s/autocomplete.html' % name,
                'autocomplete_light/autocomplete.html',
            ]

        self.request = None

    def get_absolute_url(self):
        """
        Return the absolute url for this channel, using
        autocomplete_light_channel url
        """
        return urlresolvers.reverse('autocomplete_light_channel', args=(
            self.__class__.__name__,))

    def as_dict(self):
        """
        Return a dict of variables for this channel, it is used by javascript.
        """
        return {
            'url': self.get_absolute_url(),
            'name': self.__class__.__name__
        }

    def init_for_request(self, request, *args, **kwargs):
        """
        Set self.request, self.args and self.kwargs, useful in query_filter.
        """
        self.request = request
        self.args = args
        self.kwargs = kwargs

    def query_filter(self, results):
        """
        Filter results using the request.

        By default this will expect results to be a queryset, and will filter
        it with self.search_field + '__icontains'=self.request['q'].
        """
        q = self.request.GET.get('q', None)

        if q:
            kwargs = {"%s__icontains" % self.search_field: q}
            results = results.filter(**kwargs)

        return results

    def values_filter(self, results, values):
        """
        Filter results based on a list of values.

        By default this will expect values to be an iterable of model ids, and
        results to be a queryset. Thus, it will return a queryset where pks are
        in values.
        """
        results = results.filter(pk__in=values)
        return results

    def get_queryset(self):
        """
        Return a queryset for the channel model.
        """
        return self.model.objects.all()

    def get_results(self, values=None):
        """
        Return an iterable of result to display in the autocomplete box.

        By default, it will:

        - call self.get_queryset(),
        - call values_filter() if values is not None,
        - call query_filter() if self.request is set,
        - call order_results(),
        - return a slice from offset 0 to self.limit_results.
        """
        results = self.get_queryset()

        if values is not None:
            # used by the widget to prerender existing values
            results = self.values_filter(results, values)

        elif self.request:
            # used by the autocomplete
            results = self.query_filter(results)

        return self.order_results(results)[0:self.limit_results]

    def order_results(self, results):
        """
        Return the result list after ordering.

        By default, it expects results to be a queryset and order it by
        search_field.
        """
        return results.order_by(self.search_field).distinct()

    def are_valid(self, values):
        """
        Return True if the values are valid.

        By default, expect values to be a list of object ids, return True if
        all the ids are found in the queryset.
        """
        return self.get_queryset().filter(pk__in=values).count() == len(values)

    def result_as_html(self, result, extra_context=None):
        """
        Return the html representation of a result for display in the deck
        and autocomplete box.

        By default, render result_template with channel and result in the
        context.
        """
        context = {
            'channel': self,
            'result': result,
            'value': self.result_as_value(result),
        }
        context.update(extra_context or {})
        return loader.render_to_string(self.result_template, context)

    def result_as_value(self, result):
        """
        Return the value that should be set to the widget field for a result.

        By default, return result.pk.
        """
        return result.pk

    def render_autocomplete(self):
        """
        Render the autocomplete suggestion box.

        By default, render self.autocomplete_template with the channel in the
        context.
        """
        return loader.render_to_string(self.autocomplete_template, {
            'channel': self,
        })



from django import template

register = template.Library()


@register.filter
def autocomplete_light_result_as_html(result, channel):
    """Return channel.result_as_html for result and channel."""
    return channel.result_as_html(result)



"""
A couple of helper functions to help enabling AutocompleteWidget in ModelForms.
"""
from django.forms.models import modelform_factory as django_modelform_factory
from django.db.models import ForeignKey, OneToOneField

from .widgets import AutocompleteWidget
from .registry import registry

__all__ = ['get_widgets_dict', 'modelform_factory']


def get_widgets_dict(model, autocomplete_exclude=None):
    """
    Return a dict of field_name: widget_instance for model that is compatible
    with Django.

    autocomplete_exclude
        the list of model field names to ignore

    Inspect the model's field and many to many fields, calls
    registry.channel_for_model to get the channel for the related model. If a
    channel is returned, then an AutocompleteWidget will be spawned using this
    channel.

    The dict is usable by ModelForm.Meta.widgets. In django 1.4, with
    modelform_factory too.
    """
    if autocomplete_exclude is None:
        autocomplete_exclude = []

    widgets = {}

    for field in model._meta.fields:
        if field.name in autocomplete_exclude:
            continue

        if not isinstance(field, (ForeignKey, OneToOneField)):
            continue

        channel = registry.channel_for_model(field.rel.to)
        if not channel:
            continue

        widgets[field.name] = AutocompleteWidget(channel_name=channel.__name__,
            max_items=1)

    for field in model._meta.many_to_many:
        if field.name in autocomplete_exclude:
            continue

        channel = registry.channel_for_model(field.rel.to)
        if not channel:
            continue

        widgets[field.name] = AutocompleteWidget(channel_name=channel.__name__)

    return widgets


def modelform_factory(model, autocomplete_exclude=None,
    **kwargs):
    """
    Wraps around Django's django_modelform_factory, using get_widgets_dict.

    Basically, it will use the dict returned by get_widgets_dict in order and
    pass it to django's modelform_factory, and return the resulting modelform.
    """

    widgets = get_widgets_dict(model,
        autocomplete_exclude=autocomplete_exclude)
    widgets.update(kwargs.pop('widgets', {}))
    kwargs['widgets'] = widgets

    return django_modelform_factory(model, **kwargs)



"""
``autocomplete_light.contrib.generic_m2m`` couples django-autocomplete-light
with django-generic-m2m.

Generic many to many are supported since 0.5. It depends on `django-generic-m2m
<http://django-generic-m2m.rtfd.org>`_ external apps. Follow django-generic-m2m
installation documentation, but at the time of writing it barely consists of
adding the ``genericm2m`` to ``INSTALLED_APPS``, and adding a field to models
that should have a generic m2m relation. So, kudos to the maintainers of
django-generic-m2m, fantastic app, use it for generic many to many relations.

See examples in ``test_project/generic_m2m_example``.
"""
from genericm2m.models import RelatedObjectsDescriptor

from ..generic import GenericForeignKeyField, GenericModelForm


class GenericModelForm(GenericModelForm):
    """
    Extension of autocomplete_light.GenericModelForm, that handles
    genericm2m's RelatedObjectsDescriptor.
    """

    def __init__(self, *args, **kwargs):
        """
        Add related objects to initial for each generic m2m field.
        """
        super(GenericModelForm, self).__init__(*args, **kwargs)

        for name, field in self.generic_m2m_fields():
            related_objects = getattr(self.instance, name).all()
            self.initial[name] = [x.object for x in related_objects]

    def generic_m2m_fields(self):
        """
        Yield name, field for each RelatedObjectsDescriptor of the model of
        this ModelForm.
        """
        for name, field in self.fields.items():
            if not isinstance(field, GenericManyToMany):
                continue

            model_class_attr = getattr(self._meta.model, name, None)
            if not isinstance(model_class_attr, RelatedObjectsDescriptor):
                continue

            yield name, field

    def save(self, commit=True):
        """
        Sorry guys, but we have to force commit=True and call save_m2m() right
        after.

        The reason for that is that Django 1.4 kind of left over cases where we
        wanted to override save_m2m: it enforces its own, which does not care
        of generic_m2m of course.
        """
        model = super(GenericModelForm, self).save(True)
        self.save_m2m()
        return model

    def save_m2m(self):
        """
        Save selected generic m2m relations.
        """
        if hasattr(super(GenericModelForm, self), 'save_m2m'):
            # forward compatibility:
            # Django's ModelForm **should** allow a user to overload the
            # save_m2m method. As of Django 1.4, it still enforces it's own
            # save_m2m function...
            super(GenericModelForm, self).save_m2m()

        for name, field in self.generic_m2m_fields():
            model_attr = getattr(self.instance, name)
            selected_relations = self.cleaned_data.get(name, [])

            for related in model_attr.all():
                if related.object not in selected_relations:
                    model_attr.remove(related)

            for related in selected_relations:
                model_attr.connect(related)


class GenericManyToMany(GenericForeignKeyField):
    """
    Simple form field that converts strings to models.
    """
    def prepare_value(self, value):
        if hasattr(value, '__iter__'):
            return [super(GenericManyToMany, self).prepare_value(v) \
                for v in value]
        return super(GenericManyToMany, self).prepare_value(value)

    def to_python(self, value):
        if hasattr(value, '__iter__'):
            return [super(GenericManyToMany, self).to_python(v) for v in value]
        return super(GenericManyToMany, self).to_python(value)



from django import http
from django.views import generic

import autocomplete_light

__all__ = ['ChannelView']


class ChannelView(generic.View):
    """Simple view that routes the request to the appropriate channel."""

    def get(self, request, *args, **kwargs):
        """
        Return an HttpResponse with the return value of
        channel.render_autocomplete().

        This view is called by the autocomplete script, it is expected to
        return the rendered autocomplete box contents.

        To do so, it gets the channel class from the registry, given the url
        keyword argument channel, that should be the channel name.

        Then, it instanciates the channel with no argument as usual, and calls
        channel.init_for_request, passing all arguments it recieved.

        Finnaly, it makes an HttpResponse with the result of
        channel.render_autocomplete(). The javascript will use that to fill the
        autocomplete suggestion box.
        """
        channel_class = autocomplete_light.registry[kwargs['channel']]
        channel = channel_class()
        channel.init_for_request(request, *args, **kwargs)
        return http.HttpResponse(channel.render_autocomplete())

    def post(self, request, *args, **kwargs):
        """
        Just proxy channel.post().

        This is the key to communication between the channel and the widget in
        javascript. You can use it to create results and such.
        """
        channel_class = autocomplete_light.registry[kwargs['channel']]
        channel = channel_class()
        return channel.post(request, *args, **kwargs)



from django.contrib.contenttypes.models import ContentType

from autocomplete_light.generic import GenericForeignKeyField
from .base import ChannelBase

__all__ = ['GenericChannelBase']


class GenericChannelBase(ChannelBase):
    """
    Wraps around multiple querysets, from multiple model classes, rather than
    just one.

    This is also interresting as it overrides **all** the default model logic
    from ChannelBase. Hell, you could even copy it and make your
    CSVChannelBase, a channel that uses a CSV file as backend. But only if
    you're really bored or for a milion dollars.
    """

    def result_as_value(self, result):
        """
        Rely on GenericForeignKeyField to return a string containing the
        content type id and object id of the result.

        Because this channel is made for that field, and to avoid code
        duplication.
        """
        field = GenericForeignKeyField()
        return field.prepare_value(result)

    def order_results(self, results):
        """
        Return results, **without** doing any ordering.

        In most cases, you would not have to override this method as querysets
        should be ordered by default, based on model.Meta.ordering.
        """
        return results  # can't order because don't know what model type

    def get_results(self, values=None):
        """
        Return results for each queryset returned by get_querysets().

        Note that it limits each queryset's to self.limit_result. If you want a
        maximum of 12 suggestions and have a total of 4 querysets, then
        self.limit_results should be set to 3.
        """
        for model, queryset in self.get_querysets().items():
            if values is not None:
                queryset = self.values_filter(queryset, values)

            elif self.request:
                queryset = self.query_filter(queryset)

            for result in self.order_results(queryset)[0:self.limit_results]:
                yield result

    def are_valid(self, values):
        """
        Return True if it can find all the models refered by values.
        """
        for value in values:
            # some nice Q action could be done here
            content_type_id, object_id = value.split('-')
            model = ContentType.objects.get_for_id(
                content_type_id).model_class()

            if not model.objects.filter(pk=object_id).count():
                return False

        return True

    def values_filter(self, results, values):
        """
        Filter out any result from results that is not refered to by values.
        """
        ctype = ContentType.objects.get_for_model(results.model).pk
        ids = [x.split('-')[1] for x in values \
            if int(x.split('-')[0]) == ctype]
        return results.filter(pk__in=ids)



"""
Provide tools to enable nice autocompletes in your Django project.
"""
from .registry import ChannelRegistry, registry, register, autodiscover
from .channel import ChannelBase, JSONChannelBase, RemoteChannelBase, \
    GenericChannelBase
from .widgets import AutocompleteWidget
from .forms import get_widgets_dict, modelform_factory
from .generic import GenericModelForm, GenericForeignKeyField
